#/bin/bash
# tar-wrap.sh - bash tar wrapper for xarchive frontend
# Copyright (C) 2005 Lee Bigelow <ligelowbee@yahoo.com> 
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

# set up exit status variables
E_UNSUPPORTED=65

# Supported file extentions for tar 
TAR_EXTS="tar"
GZIP_EXTS="tar.gz tgz"
BZIP2_EXTS="tar.bz tbz tar.bz2 tbz2"
COMPRESS_EXTS="tar.z"

# Program to wrap
TAR_PROG="tar"

# setup variables opt and archive.
# the shifting will leave the files passed as
# all the remaining args "$@"
opt="$1"
shift 1
archive="$1"
shift 1

# set up compression variables for our compression functions. 
# translate archive name to lower case for pattern matching.
# use compressor -c option to output to stdout and direct it where we want
lc_archive="$(echo $archive|tr [:upper:] [:lower:])"
for ext in $GZIP_EXTS; do
    if [ $(expr match "$lc_archive" ".*\."$ext"$") -gt 0 ]; then
        DECOMPRESS="gzip -dc"
        COMPRESS="gzip -c"
        TAR_COMPRESS_OPT="--use-compress-prog=gzip"
    fi
done
for ext in $BZIP2_EXTS; do
    if [ $(expr match "$lc_archive" ".*\."$ext"$") -gt 0 ]; then
        DECOMPRESS="bzip2 -dc" 
        COMPRESS="bzip2 -c"
        TAR_COMPRESS_OPT="--use-compress-prog=bzip2"
    fi
done
for ext in $COMPRESS_EXTS; do
    if [ $(expr match "$lc_archive" ".*\."$ext"$") -gt 0 ]; then
        DECOMPRESS="uncompress -dc" 
        COMPRESS="compress -c"
        TAR_COMPRESS_OPT="--use-compress-prog=compress"
    fi
done

# Command line options for prog functions
# open and extract can use tar's cmd line option
# add, new, and remove need to decompress the tar first
# do their thing, than recompress the tar.
OPEN_OPTS="$TAR_COMPRESS_OPT -tvf"
EXTRACT_OPTS="$TAR_COMPRESS_OPT -xf"
ADD_OPTS="-rf"
NEW_OPTS="-cf"
REMOVE_OPTS="--delete -f"

# Compression functions
decompress_func()
{
    if [ "$DECOMPRESS" ]; then 
        tmpname="$(mktemp -t tartmp.XXXXXX)"
        if [ -f "$archive" ]; then 
            $DECOMPRESS "$archive" > "$tmpname" 
        fi
        # store old name for when we recompress
        oldarch="$archive"
        # change working file to decompressed tmp 
        archive="$tmpname"
    fi
}

compress_func()
{
    if [ "$COMPRESS" ] && [ "$oldarch" ]; then
        [ -f "$oldarch" ] && rm "$oldarch"
        if $COMPRESS "$archive" > "$oldarch"; then
            rm "$archive"
        fi
    fi
}

# the option switches
case "$opt" in
    -i) # info: output supported extentions for progs that exist
        if [ "$(which $TAR_PROG)" ]; then
            for ext in $TAR_EXTS; do
                printf "%s;" $ext
            done
            for ext in $GZIP_EXTS; do
                if [ "$(which gzip)" ]; then
                    printf "%s;" $ext
                else
                    echo gzip not found > /dev/stderr 
                    echo extention $ext ignored > /dev/stderr 
                fi
            done
            for ext in $BZIP2_EXTS; do
                if [ "$(which bzip2)" ]; then
                    printf "%s;" $ext
                else
                    echo bzip2 not found > /dev/stderr 
                    echo extention $ext ignored > /dev/stderr
                fi
            done
            for ext in $COMPRESS_EXTS; do
                if [ "$(which compress)" ] && [ "$(which uncompress)" ]; then
                    printf "%s;" $ext
                else
                    echo compress and uncompress not found > /dev/stderr 
                    echo extention $ext ignored > /dev/stderr
                fi
            done
        else
            echo command $TAR_PROG not found > /dev/stderr 
            echo extentions $TAR_EXTS ignored > /dev/stderr
        fi
        printf "\n"
        exit
        ;;

    -o) # open: mangle output of tar cmd for xarchive 
        # format of tar output:
# lrwxrwxrwx USR/GRP       0 2005-05-12 00:32:03 file -> /path/to/link
# -rw-r--r-- USR/GRP    6622 2005-04-22 12:29:14 file 
        
        ATTR=0; USR=1; GRP=2; SIZE=3; DATE=4; TIME=5; NAME=6; LINK=7

        $TAR_PROG $OPEN_OPTS "$archive" | while read dataline; do
            i=0
            a[LINK]="" # clear link, only set if link exists 
            for element in $dataline; do
                if [ $i -lt $NAME ]; then      #NAME and LINK handled below 
                    if [ $i -eq $USR ]; then   #tar outputs usr/group, split it
                        a[i]=${element%%/*}
                        let i++
                        a[i]=${element##*/}
                    else
                        a[i]=$element          #others can be normally assigned
                    fi
                elif [ "$element" = "->" ]; then #Test for symlnk past NAME
                        a[LINK]=1
                        break
                fi
                let i++
            done
            # use the whole dataline and strip everything before date and a
            # space to get name and maybe link
            tmpname="${dataline##*??:??:?? }" 
            # strip all after and including " -> " for NAME
            a[NAME]="${tmpname%% -> *}" 
            # strip all before and including " -> " if link for LINK 
            [ "${a[LINK]}" ] && a[LINK]="${tmpname##* -> }" 
            # output for xarchive
            echo "${a[NAME]};${a[SIZE]};${a[ATTR]};${a[USR]};${a[GRP]};${a[DATE]};${a[TIME]};${a[LINK]}" 
        done
        exit
        ;;

    -a) # add to archive passed files
        # we only want to add the file's basename, not
        # the full path so...
        decompress_func
        while [ "$1" ]; do
            cd "$(dirname "$1")"
            $TAR_PROG $ADD_OPTS "$archive" "$(basename "$1")"
            wrapper_status=$?
            shift 1
        done
        compress_func
        exit $wrapper_status
        ;;

    -n) # new: create new archive with passed files 
        # create will only be passed the first file, the
        # rest will be "added" to the new archive
        decompress_func
        cd "$(dirname "$1")"
        $TAR_PROG $NEW_OPTS "$archive" "$(basename "$1")"
        wrapper_status=$?
        compress_func
        exit $wrapper_status
        ;;

    -r) # remove: from archive passed files 
        decompress_func
        $TAR_PROG $REMOVE_OPTS "$archive" "$@"
        wrapper_status=$?
        compress_func
        exit $wrapper_status
        ;;

    -e) # extract: from archive passed files 
        # xarchive will put is the right extract dir
        # so we just have to extract.
        $TAR_PROG $EXTRACT_OPTS "$archive" "$@" 
        exit 
        ;;

    -v) # view: from archive passed files 
        exit E_UNSUPPORTED
        ;;

     *) echo "error, option $opt not supported"
        echo "use one of these:" 
        echo "-i                #info" 
        echo "-o archive        #open" 
        echo "-a archive files  #add" 
        echo "-n archive file   #new" 
        echo "-r archive files  #remove" 
        echo "-e archive files  #extract" 
        echo "-v archive file   #view"  
        exit
esac
